/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Adressverwaltung
Name        : addressConsoleIO.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-19v1.0
Description : Adressverwaltung
              Adressen aus der Konsole einlesen, bestehende ausgeben
==================================================================================================*/
#include <stdio.h>
#include <string.h>
#include "address.h"
#include "addressConsoleIO.h"

void showMenu(int dispLargeMenu)
{
    char menu = ' '; // buffer for key, user menu choice
    int listAfterSort = TRUE;

    do // Menu (until Q pressed)
    {
        
        if (dispLargeMenu) {
            // show Main Menu Desc (large)
            showMainMenuLarge();
            dispLargeMenu = FALSE;
        } else {
            // show Main Menu Desc (small)
            showMainMenuSmall();
        }
        printf(" » Please choose: ");

        while (!isalnum(menu = getchar())); // Solange einlesen bis Zahl oder Buchstabe
        menu = toupper(menu);

        switch(menu){
        case 'N': // Insert a new address
            addAddressGUI();
            break;
        case 'L': // List stored addresses
            listAddressGUI();
            break;
        case 'C': // Clear stored addresses
            clearAddressGUI();
            break;
        case 'R': // Read addresses from file
            readAddrFileGUI();
            break;
        case 'S': // Save addresses to file
            saveAddrFileGUI();
            break;
        case 'M': // Show Main Menu
            dispLargeMenu = TRUE;
            break;
        case '0': // Show List after Sort
            if (listAfterSort == TRUE) {
                listAfterSort = FALSE;
            } else {
                listAfterSort = TRUE;
            }
            break;
        case '1': // Sort addresses by name & List them
            sortAddressGUI(NAME);
            if (listAfterSort) { listAddressGUI(); }
            break;
        case '2': // Sort addresses by street & List them
            sortAddressGUI(STREET);
            if (listAfterSort) { listAddressGUI(); }
            break;
        case '3': // Sort addresses by zip & List them
            sortAddressGUI(ZIP);
            if (listAfterSort) { listAddressGUI(); }
            break;
        case '4': // Sort addresses by city & List them
            sortAddressGUI(CITY);
            if (listAfterSort) { listAddressGUI(); }
            break;
        }
    } while(menu != 'Q');
}

void showMainMenuLarge(void)
{
        system("cmd.exe /c cls"); // Cygwin hack
        printf("╔═════════════════════════════════════════════════════════════════════════════╗\n");
        printf("║        _      _    _                 __  __                                 ║\n");
        printf("║       /_\\  __| |__| |_ _ ___ ______ |  \\/  |__ _ _ _  __ _ __ _ ___ _ _     ║\n");
        printf("║      / _ \\/ _` / _` | '_/ -_|_-<_-< | |\\/| / _` | ' \\/ _` / _` / -_) '_|    ║\n");
        printf("║     /_/ \\_\\__,_\\__,_|_| \\___/__/__/ |_|  |_\\__,_|_||_\\__,_\\__, \\___|_|      ║\n");
        printf("║                                                           |___/             ║\n");
        printf("╠═════════════════════════════════════════════════════════════════════════════╣\n");    
        printf("║   Select an option                                                          ║\n");
        printf("║     N : Insert a new address         1 : Sort addresses by name             ║\n");
        printf("║     L : List stored addresses        2 : Sort addresses by street           ║\n");
        printf("║     C : Clear stored addresses       3 : Sort addresses by zip              ║\n");
        printf("║                                      4 : Sort addresses by city             ║\n");
        printf("║     R : Read addresses from file                                            ║\n");
        printf("║     S : Save addresses to file       Q : Quit            © 2012 Felix Rohrer║\n");
        printf("╚═════════════════════════════════════════════════════════════════════════════╝\n");    
}

void showMainMenuSmall(void)
{
        printf("\n");
        printf("╔════════╦═════════╦══════════╦═════════════╦═════════════╦═════════╦═════════╗\n");
        printf("║ N: New ║ L: List ║ C: Clear ║ R: ReadFile ║ S: SaveFile ║ M: Menu ║ Q: Quit ║\n");
        printf("╠════════╩═════════╬══════════╩═════════╦═══╩═════════════╬═════════╩═════════╣\n");
        printf("║ 1: sort by name  ║ 2: sort by street  ║ 3: sort by zip  ║ 4: sort by city   ║\n");
        printf("╚══════════════════╩════════════════════╩═════════════════╩═══════════════════╝\n");
}

void addAddressGUI(void)
{
    int res;                     // errorcode handler
    char firstname[MAXSTRLEN+1]; // Buffer for Firstname
    char lastname[MAXSTRLEN+1];  // Buffer for Lastname
    char street[MAXSTRLEN+1-5];  // Buffer for Street
    char streetNbr[5+1];         // Buffer for Street Number
    unsigned int zip;            // Buffer for Zip
    char city[MAXSTRLEN+1];      // Buffer for City
    size_t length;               // Lenght of Street, combine Street + StreetNbr
    
    system("cmd.exe /c cls"); // Cygwin hack
    printf("┌─────────────────────────────────────────────────────────────────────────────┐\n");
    printf("│ Add new address                                                             │\n");
    printf("├─────────────────────────────────────────────────────────────────────────────┘\n");
    printf("│ Firstname:  ");
    scanf("%s", firstname);
    printf("│ Lastname:   ");
    scanf("%s", lastname);
    printf("│ Street:     ");
    scanf("%s", street);
    printf("│ Street Nbr: ");
    scanf("%s", streetNbr);
    printf("│ Zip:        ");
    scanf("%i", &zip);
    printf("│ City:       ");
    scanf("%s", city);
    printf("├──────────────────────────────────────────────────────────────────────────────\n");
    
    // combine street & streetnumber
    length = strlen(street);
    street[length] = ' ';
    street[length+1] = '\0';
    strcat(street, streetNbr);
    // add new addr.
    res = addAddressValues(lastname, firstname, street, zip, city);
    if (res != SUCCESS) {
        printf("│ ERROR: Could not add new address! (Err: %i)\n", res);
    } else {
        printf("│ New address added successfully.\n");
    }
    printf("└──────────────────────────────────────────────────────────────────────────────\n");
}

void saveAddrFileGUI(void)
{
    err_t res;            // errorcode handler
    char path[PATH_SIZE]; // filepath, filename buffer
    int count = 0;        // counter of addresses
    
    system("cmd.exe /c cls"); // Cygwin hack
    printf("┌─────────────────────────────────────────────────────────────────────────────┐\n");
    printf("│ Save to file                                                                │\n");
    printf("├─────────────────────────────────────────────────────────────────────────────┘\n");
    printf("│ Please enter filename: ");
    scanf("%s", path);
    res = saveAddrFile(path, &count); // Save AddressList to a File
    if (res != SUCCESS) {
        printf("│ ERROR: Could not write file! (Err: %i)\n", res);
    } else {
        printf("│ Save done. %i addresses saved into file '%s'\n", count, path);
    }
    printf("└──────────────────────────────────────────────────────────────────────────────\n");
}

void readAddrFileGUI(void)
{
    err_t res;            // errorcode handler
    char path[PATH_SIZE]; // filepath, filename buffer
    int count = 0;        // counter of addresses
    
    system("cmd.exe /c cls"); // Cygwin hack
    printf("┌─────────────────────────────────────────────────────────────────────────────┐\n");
    printf("│ Load from file                                                              │\n");
    printf("├─────────────────────────────────────────────────────────────────────────────┘\n");
    printf("│ Please enter filename: ");
    scanf("%s", path);
    res = readAddrFile(path, &count); // Read Addresses from a File and add them to the current AddressList
    if (res != SUCCESS) {
        printf("│ ERROR: Could not read file '%s'! (Err: %i)\n", path, res);
    } else {
        printf("│ Load done. %i addresses loaded from file '%s'.\n", count, path);
    }
    printf("└──────────────────────────────────────────────────────────────────────────────\n");
}

void sortAddressGUI(sortmode mode)
{
    err_t res; // errorcode handler
    
    res = sortAddress(mode); // SortAddress by SortMode
    if (res == SUCCESS) {
        printf(" » Address List successfully sorted...\n");
    } else {
        printf(" » ERROR: Address List could not be sorted! (Err: %i)\n", res);
    }
}

void clearAddressGUI(void)
{
    err_t res; // errorcode handler
    
    res = clearAddress(); // free & re-malloc memory
    if (res == SUCCESS) {
        printf(" » Address List cleared...\n");
    } else {
        printf(" » ERROR: Address List could not be cleared! (Err: %i)\n", res);
    }
}

void displayAddr(int count, address_t* addr)
{
    char nameBuff[2*MAXSTRLEN+1]; // Buffer for Lastname + Firstname
    
    sprintf(nameBuff, "%s %s", addr->lastname, addr->firstname);
    printf("│ %4i %-25.24s%-25.24s%-5d%.16s\n",
            count,
            nameBuff,
            addr->street,
            addr->zip,
            addr->city);
}

void listAddressGUI(void)
{
    system("cmd.exe /c cls"); // Cygwin hack
    printf("┌─────────────────────────────────────────────────────────────────────────────┐\n");
    printf("│ Address List                                                                │\n");
    printf("├─────────────────────────────────────────────────────────────────────────────┘\n");
    printf("│ %4s %-25s%-25s%-5s%s\n", "#", "Name", "Street", "Zip", "City");
    listAddress(displayAddr); // ListAddress with CallBack Function displayAddr -> Console Stdout
    printf("└──────────────────────────────────────────────────────────────────────────────\n");
}

