/**=================================================================================================
Hochschule Luzern - Technik & Architektur                                   Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                     http://hslu.ximit.ch
----------------------------------------------------------------------------------------------------
Project     : Adressverwaltung
Name        : addressFileIO.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-19v1.0
Description : Adressverwaltung
              Adressen in eine Datei (csv) speichern oder aus einer Datei importieren.
==================================================================================================*/
#include <stdio.h>
#include <string.h>
#include "address.h"
#include "addressFileIO.h"

#define BUFFER_SIZE 2048 // File Line Buffer Size

err_t lineParser(char* txtLine)
{
    char lastname[MAXSTRLEN+1];  // buffer for lastname
    char firstname[MAXSTRLEN+1]; // buffer for firstname
    char street[MAXSTRLEN+1];    // buffer for street
    int zip;                     // buffer for zip
    char city[MAXSTRLEN+1];      // buffer for city
    char* txtPart;               // buffer for text token

    // init, set default to empty
    strcpy(firstname, "\0");
    strcpy(lastname, "\0");
    strcpy(street, "\0");
    zip = -1;
    strcpy(city, "\0");
    
    // Reads all address tokens out of txtLine
    // lastname
    txtPart = strtok(txtLine, "\t"); // only first read needs the txtLine
    if (txtPart != NULL && (strlen(txtPart)<MAXSTRLEN)) {
        strcpy(lastname, txtPart);
    } else { return FILEREADERROR; }
    
    // firstname
    txtPart = strtok(NULL, "\t");
    if (txtPart != NULL && (strlen(txtPart)<MAXSTRLEN)) {
        strcpy(firstname, txtPart);
    } else { return FILEREADERROR; }

    // street
    txtPart = strtok(NULL, "\t");
    if (txtPart != NULL && (strlen(txtPart)<MAXSTRLEN)) {
        strcpy(street, txtPart);
    } else { return FILEREADERROR; }

    // zip
    txtPart = strtok(NULL, "\t");
    if (txtPart != NULL && (strlen(txtPart)<MAXSTRLEN)) {
        zip = atoi(txtPart);
    } else { return FILEREADERROR; }

    // city
    txtPart = strtok(NULL, "\r"); // \r to remove trailing cr / newline
    if (txtPart != NULL && (strlen(txtPart)<MAXSTRLEN)) {
        strcpy(city, txtPart);
    } else { return FILEREADERROR; }
        
    // add new Address
    return addAddressValues(lastname, firstname, street, zip, city);
}

err_t writeFile(char* path, addressList_t* addrList, err_t (*formatCSVfn)(address_t*, char*), int* num)
{
    int res;                // errorcode handler
    int count;              // counter of addresses
    int i;                  // tmp counter
    FILE *file = NULL;      // file handler
    char line[BUFFER_SIZE]; // buffer for one line
    
    // create new file (overwrite existing one)
    file = fopen(path, "w+");
    if (file == NULL) {
        return FILECREATEERROR;
    }
    // write file
    count = addrList->size;
    *num = count; // save number of addresses for output
    for (i=0; i<count; i++) {
        res = (*formatCSVfn)(&(addrList->addr[i]), line); // Use CallBack-Func. to Format the Export
        if (res != SUCCESS || fputs(line, file) == EOF) {
            free(line);
            fclose(file);
            return res;
        }
    }
    // close the file, define return status
    if (fclose(file) != 0) {
        res = FILECLOSEERROR;
    } else {
        res = SUCCESS;
    }
    return res;
}

err_t readFile(char* path, addressList_t* addrList, err_t (*lineParserFn)(char*), int* num)
{
    err_t res;              // errorcode handler
    FILE *file = NULL;      // file handler
    char line[BUFFER_SIZE]; // buffer file reading
    int count = 0;          // Counter for addresses
    
    // Open the file (read-only)
    file = fopen(path, "r");
    if (file == NULL) {
        return FILENOTFOUNDERROR;
    }

    // Reads until no more lines are available in the csv
    while(fgets(line, sizeof(line), file) != NULL) {
        count++;
        // try to parse and process the file (Use CallBack-Func.)
        res = (*lineParserFn)(line);
        // in case of error, close the file and abort with error code
        if (res != SUCCESS) {
            fclose(file);
            file = NULL;
            return res;
        }
    }
    
    // close the file
    if (fclose(file) != 0) {
        res = FILECLOSEERROR;
    } else {
        res = SUCCESS;
    }
    *num = count; // save back counter value
    return res;   
}

err_t formatCSVTab(address_t* addr, char* line){
    // format string for csv export
    sprintf(line, "%s\t%s\t%s\t%i\t%s\r\n",
            addr->lastname,
            addr->firstname,
            addr->street,
            addr->zip,
            addr->city);
    return SUCCESS;
}

err_t formatCSVSemicolon(address_t* addr, char* line){
    // format string for csv export
    sprintf(line, "%s;%s;%s;%i;%s\r\n",
            addr->lastname,
            addr->firstname,
            addr->street,
            addr->zip,
            addr->city);
    return SUCCESS;
}
